@@ -188,9 +188,6 @@ def gausshermitebin(x,par,binsize=1.0):
     
     nx = x.shape[0]
     npar = par.shape[1]
-
-    #import pdb; pdb.set_trace()
-
     
     # HH hermite polynomials
     # GHfunc  the output value
@@ -366,7 +363,6 @@ def gausshermitebin(x,par,binsize=1.0):
     
     return GHfunc
 
-
 def ghwingsbin(x,par,binsize=1.0,Wproftype=1):
     """This function returns profile wings with the APOGEE LSF or PSF.
 
@@ -721,6 +717,70 @@ def unpack_ghlsf_params(lsfarr):
     out['Wcoefs'] = Wcoefs
     return out
 
+def repack_ghlsf_params(params):
+    """
+    Repack the APOGEE Gauss-Hermite LSF dictionary into the parameter array.
+
+    Parameters
+    ----------
+    params : dictionary
+        Dictionary with unpacked parameters and parameter values: 
+          binsize: The width of a pixel in X-units
+          Xoffset: An additive x-offset; used for GH parameters that vary globally
+          Horder: The highest Hermite order
+          Porder: Polynomial order array for global variation of each LSF parameter
+          GHcoefs: Polynomial coefficients for sigma and the Horder Hermite parameters
+          Wproftype: Wing profile type
+          nWpar: Number of wing parameters
+          WPorder: Polynomial order for the global variation of each wing parameter          
+          Wcoefs: Polynomial coefficients for the wings parameters
+
+    Returns
+    -------
+    lsfarr : array
+        Array of APOGEE Gauss-Hermite LSF parameters.
+
+    Examples
+    --------
+    .. code-block:: python
+
+         lsfarr = repack_ghlsf_params(params)
+
+    """
+    
+    # If 2D then iterate over orders and return list
+    if type(params) is list:
+        out = []
+        for p in params:
+            out.append(repack_ghlsf_params(p))
+        out = np.array(out).T
+        return out
+
+    # Process single chip parameter dictionary
+    lsfarr = np.array([],float)
+    # binsize: The width of a pixel in X-units
+    lsfarr = np.append(lsfarr,params['binsize'])
+    # X0: An additive x-offset; used for GH parameters that vary globally
+    lsfarr = np.append(lsfarr,params['Xoffset'])
+    # Horder: The highest Hermite order
+    lsfarr = np.append(lsfarr,params['Horder'])
+    # Porder: Polynomial order array for global variation of each LSF parameter
+    lsfarr = np.append(lsfarr,params['Porder'])
+    # GHcoefs: Polynomial coefficients for sigma and the Horder Hermite parameters
+    for i in range(len(params['Porder'])):
+        ghcoef = params['GHcoefs'][i,:]
+        lsfarr = np.append(lsfarr,ghcoef[:params['Porder'][i]+1])
+    # Wproftype: Wing profile type
+    lsfarr = np.append(lsfarr,params['Wproftype'])
+    # nWpar: Number of wing parameters
+    lsfarr = np.append(lsfarr,params['nWpar'])
+    # WPorder: Polynomial order for the global variation of each wing parameter
+    lsfarr = np.append(lsfarr,params['WPorder'])
+    # Wcoefs: Polynomial coefficients for the wings parameters
+    for i in range(len(params['WPorder'])):
+        wcoef = params['Wcoefs'][i,:]
+        lsfarr = np.append(lsfarr,wcoef[:params['WPorder'][i]+1])
+    return lsfarr
 
 # Base class for representing LSF (line spread function)
 class Lsf:
@@ -762,27 +822,30 @@ class Lsf:
             norder = 1
         else:
             npix,norder = wave.shape        
-        self.ndim = wave.ndim
-        self.npix = npix
-        self.norder = norder        
         # Get number of pixels per order
-        if self.norder>1:
+        if self.norder>1 or wave.ndim==2:
             numpix = np.zeros(norder,int)
             for i in range(norder):
                 gdpix, = np.where((wave[:,i]>0) & np.isfinite(wave[:,i]))
-                numpix[i] = np.max(gdpix)+1
+                if len(gdpix)>0:
+                    numpix[i] = np.max(gdpix)+1
+                else:
+                    numpix[i] = 0
             self.numpix = numpix
         else:
             gdpix, = np.where((wave>0) & np.isfinite(wave))
-            numpix = np.max(gdpix)+1
+            if len(gdpix)>0:
+                numpix = np.max(gdpix)+1
+            else:
+                numpix = 0
             self.numpix = [numpix]
         # Make sure pars are 2D
         self.pars = pars
         if pars is not None:
-            if pars.ndim==1:
+            if np.array(pars).ndim==1:
                 self.pars = np.atleast_2d(pars).T   # 2D with order dimension at 2nd
             else:
-                self.pars = pars
+                self.pars = np.array(pars)
         self.lsftype = lsftype
         self.xtype = xtype
         # Make sure sigma is 2D
@@ -804,31 +867,54 @@ class Lsf:
         """ Returns the Gaussian array.  Must be defined by the subclass."""
         pass
 
-
     def __getitem__(self,index):
-        """ Return a single order of the LSF object."""
+        """ Return a slice or subset of the LSF object."""
+        pass
 
-        if type(index) is not int:
-            raise IndexError('Index must be an integer')
-        if index>=self.norder:
-            raise IndexError('Index is too large')
-        # Single order, return self
-        if self.norder==1:
-            return self
-        slc = slice(0,self.numpix[index])
-        wave = self.wave[slc,index]
-        if self.pars is not None:
-            pars = self.pars[slc,index]
-        else:
-            pars = None
-        if self._sigma is not None:
-            sigma = self._sigma[slc,index]
-        else:
-            sigma = None
-        olsf = Lsf(wave=wave,pars=pars,xtype=self.xtype,lsftype=self.lsftype,
-                   sigma=sigma)
-        return olsf
+    @property
+    def size(self):
+        """ Return number of total 'good' pixels (not including buffer pixels)."""
+        return np.sum(self.numpix)
+
+    @property
+    def ndim(self):
+        """ Return the number of dimensions."""
+        return self.wave.ndim
+
+    @property
+    def npix(self):
+        """ Return the number of pixels in each order."""
+        return self.shape[0]
     
+    @property
+    def norder(self):
+        """ Return the number of orders."""
+        if self.ndim==1:
+            return 1
+        else:
+            return self.shape[1]
+
+    def __len__(self):
+        return self.norder
+
+    @property
+    def shape(self):
+        return self.wave.shape
+    
+    @property
+    def wrange(self):
+        """ Wavelength range."""
+        wr = [np.inf,-np.inf]
+        if self.wave.ndim==1:
+            w = self.wave[0:self.numpix[0]]
+            wr = [np.min(w),np.max(w)]
+        else:
+            for i in range(self.norder):
+                w = self.wave[0:self.numpix[i],i]
+                wr[0] = np.min([wr[0],np.min(w)])
+                wr[1] = np.max([wr[1],np.max(w)])            
+        return wr
+        
     def wave2pix(self,w,extrapolate=True,order=0):
         """
         Convert wavelength values to pixels using the LSF's dispersion
@@ -899,15 +985,67 @@ class Lsf:
         
         if self.wave is None:
             raise Exception("No wavelength information")
-        if self.ndim==2:
-             # Order is always the second dimension
-            wave = self.wave[:,order]
-        else:
-            wave = self.wave
-        gdwave, = np.where((wave>0) & np.isfinite(wave))
-        return utils.p2w(wave[gdwave],x,extrapolate=extrapolate)        
+        return utils.p2w(self[order].wave,x,extrapolate=extrapolate)
 
+    def dispersion(self,x=None,xtype='pixels',extrapolate=True,order=0):
+        """
+        Wavelength dispersion or step (delta wavelength) at a given pixel
+        or wavelength.  This is *always* positive.
+
+        Parameters
+        ----------
+        x : array
+           Array of pixel values at which to return the dispersion
+        xtype : str, optional
+           The type of x-value in put (either 'wave' or 'pixels').  The default
+           is 'pixels'.
+        extrapolate : bool, optional
+           Extrapolate beyond the boundaries of the wavelength solution,
+           if necessary.  True by default.
+        order : int, optional
+           The order to use if there are multiple orders.
+           The default is 0.
+              
+        Returns
+        -------
+        dw : array
+           The array of wavelengths dispersion/gradient values.
+
+        Example
+        -------
+        .. code-block:: python
+
+             dw = lsf.dispersion(x)
+
+        """
         
+        if self.wave is None:
+            raise Exception("No wavelength information")
+        wave = self[order].wave
+        dwave = np.abs(np.gradient(wave))
+        if x is None:
+            return dwave
+        if xtype.lower().find('pix')>-1:
+            abscissa = np.arange(len(wave))
+        else:
+            abscissa = x
+        dw = dln.interp(abscissa,dwave,x,kind='quadratic',assume_sorted=False,
+                        extrapolate=extrapolate,exporder=1)
+        if np.array(x).ndim==0: dw=dw[0]
+        # fix negative values when extrapolating
+        if extrapolate:
+            bad = (dw < 0)
+            if np.sum(bad)>0:
+                # set to values on the ends
+                bdlo, = np.where(x < abscissa[0])
+                if len(bdlo)>0:
+                    dw[bdlo] = dwave[0]
+                bdhi, = np.where(x > abscissa[-1])
+                if len(bdhi)>0:
+                    dw[bdhi] = dwave[-1]
+        return dw
+
+    
     # Return FWHM at some positions
     def fwhm(self,x=None,xtype='pixels',order=0):
         """
@@ -968,8 +1106,26 @@ class Lsf:
         """ Create a new copy of this LSF object."""
         return copy.deepcopy(self)
 
+    def remove_order(self,order):
+        """ Remove orders from spectrum."""
+        for c in ['wave','pars','_sigma']:
+            if hasattr(self,c) and getattr(self,c) is not None:
+                setattr(self,c,np.delete(getattr(self,c),order,axis=1))
+        self.numpix = np.delete(self.numpix,order)
+        # Single order, use 1-D arrays
+        if self.norder==1:
+            for c in ['wave','pars','_sigma']:
+                if hasattr(self,c) and getattr(self,c) is not None:
+                    setattr(self,c,getattr(self,c).flatten())
 
+    def trim(self):
+        """ Trim the LSF in wavelength."""
+        pass
 
+    def rebin(self,nbin):
+        """ Rebin the data."""
+        pass
+    
 # Class for representing Gaussian LSFs
 class GaussianLsf(Lsf):
     """
@@ -1098,12 +1254,12 @@ class GaussianLsf(Lsf):
         Parameters
         ----------
         x : array, optional
-          The x-values for which to return the Gaussian sigma values.
+           The x-values for which to return the Gaussian sigma values.
         xtype : string, optional
            The type of x-value input, either 'wave' or 'pixels'.  Default is 'pixels'.
         order : int, optional
-            The order to use if there are multiple orders.
-            The default is 0.
+           The order to use if there are multiple orders.
+           The default is 0.
         extrapolate : bool, optional
            Extrapolate beyond the dispersion solution, if necessary.
            True by default.
@@ -1111,8 +1267,8 @@ class GaussianLsf(Lsf):
         Returns
         -------
         sigma : array
-            The array of Gaussian sigma values.  The sigma values will
-             be in units of lsf.xtype.
+           The array of Gaussian sigma values.  The sigma values will
+           *always* be in units of lsf.xtype.
 
         Example
         -------
@@ -1155,12 +1311,14 @@ class GaussianLsf(Lsf):
                             coef2 = dln.poly_fit(xin[npix-10:], _sigma[npix-10:], 1)
                             bd2, nbd2 = dln.where(x > (npix-1))
                             sig[bd2] = dln.poly(x[bd2],coef2)
+
                     return sig
                         
-        # Need to calculate
+        # Need to calculate sigma from pars
         else:
             if x is None:
                 x = np.arange(self.npix)
+                xtype = 'Pixels'
             if self.pars is None:
                 raise Exception("No LSF parameters")
             # Get parameters
@@ -1192,7 +1350,7 @@ class GaussianLsf(Lsf):
         """ Clean up bad input Gaussian sigma values."""
         if self._sigma is not None:
             smlen = np.round(self.npix // 50).astype(int)
-            if smlen==0: smlen=3
+            if smlen==0: smlen=51
             _sigma = self._sigma.reshape(self.npix,self.norder)   # make 2D
             for o in range(self.norder):
                 sig = _sigma[:,o]
@@ -1201,6 +1359,14 @@ class GaussianLsf(Lsf):
                     sig[bd] = np.nan
                     smsig = dln.gsmooth(sig,smlen)
                     sig[bd] = smsig[bd]
+                gd,ngd,bd,nbd = dln.where(sig > 0.001,comp=True)
+                if nbd>0:
+                    sig[bd] = np.nan
+                    smsig = dln.gsmooth(sig,3*smlen)
+                    sig[bd] = smsig[bd]
+                gd,ngd,bd,nbd = dln.where(sig > 0.001,comp=True)
+                if nbd>0:
+                    sig[bd] = np.median(sig[gd])                    
                 if self.ndim==2:
                     self._sigma[:,o] = sig
                 else:
@@ -1301,12 +1467,12 @@ class GaussianLsf(Lsf):
           The type of x-values input, either 'wave' or 'pixels'.  The default
           is 'pixels'.
         order : int, optional
-           The order for which to retrn the LSF array.  The default is 0.
+           The order for which to return the LSF array.  The default is 0.
         original : bool, optional
            If original=True, then the LSFs are returned on the original
            wavelength scale but at the centers given in "x".
            If the LSF is desired on a completely new wavelength scale
-           (given by "x"), then orignal=False should be used instead.
+           (given by "x"), then original=False should be used instead.
 
         Returns
         -------
@@ -1322,7 +1488,7 @@ class GaussianLsf(Lsf):
         """
 
         nx = len(x)
-        # returns xsigma in units of self.xtype not necessarily xtype
+        # returns xsigma in units of self.xtype (NOTE: not necessarily *input* xtype)
         xsigma = self.sigma(x,xtype=xtype,order=order)
         
         # Get wavelength and pixel arrays
@@ -1345,6 +1511,17 @@ class GaussianLsf(Lsf):
                 dw = np.hstack((dw,dw[-1]))            
             xsigma = wsigma / dw
 
+        # Convert sigma from original pixels to new
+        #  wavelength/pixel scale, if necessary
+        else:
+            # New wavelength/pixel scale
+            if original==False:
+                origdw = self.dispersion(x,xtype='pixels',order=order)
+                #w1 = self.pix2wave(np.array(x)+1,order=order)
+                #origdw = np.abs(w1-w)
+                newdw = np.abs(np.gradient(w))
+                xsigma *= origdw / newdw
+            
         # Figure out nLSF pixels needed, +/-3 sigma
         nlsf = int(np.round(np.max(xsigma)*6))
         if nlsf % 2 == 0: nlsf+=1                   # must be odd
@@ -1364,7 +1541,252 @@ class GaussianLsf(Lsf):
         # should I use gaussbin????
         return lsf
 
+    def trim(self,w0,w1):
+        """
+        Trim the LSF in wavelength.
 
+        Parameters
+        ----------
+        w0 : float
+           Lower wavelength (in Ang) to trim.
+        w1 : float
+           Upper wavelength (in Ang) to trim.
+
+        Returns
+        -------
+        Nothing is returned.  The spectrum is trimmed in place.
+
+        Examples
+        --------
+
+        lsf.trim(5500.0,6500.0)
+
+        """
+        # Get sigma, we might need this below
+        if self.xtype.lower().find('pix')>-1:
+            sigma = np.zeros([self.npix,self.norder],float)
+            for o in range(self.norder):
+                x = np.arange(self.numpix[o])
+                sigma[:self.numpix[o],o] = self.sigma(x,order=o)
+        else:
+            sigma = None
+        # Get number of pixels per order
+        ranges = np.zeros([self.norder,2],int)-1
+        ngood = np.zeros(self.norder,int)        
+        for o in range(self.norder):
+            if self.norder==1:
+                ind, = np.where((self.wave>=w0) & (self.wave<=w1))
+            else:
+                npix = self.numpix[o]
+                ind, = np.where((self.wave[:npix,o]>=w0) & (self.wave[:npix,o]<=w1))
+            ngood[o] = len(ind)
+            if len(ind)>0:
+                ranges[o,0] = ind[0]
+                ranges[o,1] = ind[-1]
+        # Check that we have some pixels
+        goodorder, = np.where(ngood>0)
+        if len(goodorder)==0:
+            raise ValueError('No pixels left')
+        # Remove blank orders
+        blankorder, = np.where(ngood==0)
+        for o in blankorder:
+            self.remove_order(o)
+        if sigma is not None:
+            sigma = sigma[:,goodorder]
+            if len(goodorder)==1:
+                sigma = sigma.flatten()
+        ranges = ranges[goodorder,:]
+        numpix = list(ranges[:,1]-ranges[:,0]+1)
+        npix = np.max(numpix)    # new npix  
+        if len(goodorder)==1:
+            ranges = ranges.flatten()
+        # Trim arrays
+        default = {'wave':0.0,'_sigma':0.0}
+        for c in ['wave','_sigma']:
+            if hasattr(self,c) and getattr(self,c) is not None:
+                arr = getattr(self,c)
+                if self.norder==1:
+                    newarr = arr[ranges[0]:ranges[1]+1]
+                else:
+                    newarr = np.zeros([npix,self.norder],arr.dtype)
+                    newarr[:,:] = default[c]
+                    for o in range(self.norder):
+                        newvals = arr[ranges[o,0]:ranges[o,1]+1,o].copy()
+                        newarr[:len(newvals),o] = newvals
+                setattr(self,c,newarr)
+        self.numpix = numpix
+        if sigma is not None:
+            if self.norder==1:
+                sigma = sigma[ranges[0]:ranges[1]+1]
+            else:
+                for o in range(self.norder):
+                    newvals = sigma[ranges[o,0]:ranges[o,1]+1,o].copy()
+                    sigma[:len(newvals),o] = newvals
+        # Fix pars if using pixel-based values
+        #   xtype='Wave' is fine
+        if self.xtype.lower().find('pix')>-1 and hasattr(self,'pars') and self.pars is not None:
+            for o in range(self.norder):
+                x = np.arange(self.numpix[o])
+                if self.norder==1:
+                    sigma1 = sigma[:self.numpix[o]]
+                else:
+                    sigma1 = sigma[:self.numpix[o],o]
+                coef = np.polyfit(x,sigma1,self.pars.shape[0]-1)
+                self.pars[:,o] = coef[::-1]
+
+    def rebin(self,nbin,mean=False):
+        """
+        Rebin the LSF information in the spectral dimension.
+
+        Parameters
+        ----------
+        nbin : int
+            Integer number of pixels to bin.
+        mean : bool, optional
+            Take the mean instead of the default sum.
+
+        Returns
+        -------
+        Nothing is returned.  The spectrum is rebinned in place.
+
+        Examples
+        --------
+
+        lsf.rebin(2)
+
+        """
+        # Only keep complete bins and remove any excess
+        newnpix = self.npix//nbin
+        numpix = self.numpix.copy()        
+        # Bin the data
+        default = {'wave':0.0,'_sigma':0.0}
+        for c in ['wave','_sigma']:
+            if hasattr(self,c) and getattr(self,c) is not None:
+                arr = getattr(self,c)
+                if self.norder==1:
+                    arr = np.atleast_2d(arr).T
+                # Use mean for both wave and _sigma
+                newarr = np.zeros([newnpix,self.norder],arr.dtype)
+                newarr[:,:] = default[c]
+                for o in range(self.norder):
+                    arr1 = arr[:self.numpix[o],o]  # trim to good values
+                    newvals = dln.rebin(arr1,binsize=nbin)
+                    if c=='_sigma' and self.xtype.lower().find('pix')>-1:
+                        newvals /= nbin    # reduce for new binsize
+                    newarr[:len(newvals),o] = newvals
+                    numpix[o] = len(newvals)
+                if self.norder==1:
+                    newarr = newarr.flatten()
+                setattr(self,c,newarr)
+        self.numpix = numpix
+        # Fix pars if using pixel-based values
+        #   xtype='Wave' is fine
+        if self.xtype.lower().find('pix')>-1 and hasattr(self,'pars') and self.pars is not None:
+            # Since we are going from X -> X/nbin
+            # we need to scale the polynomial coefficients
+            # c0 -> c0
+            # c1 -> c1*nbin
+            # c2 -> c2*nbin**2
+            for i in range(self.pars.shape[0]):
+                self.pars[i,:] *= nbin**i
+            # Reduce sigma since our pixels are getting larget
+            self.pars /= nbin
+        
+    def __getitem__(self,index):
+        """ 
+        Return slice or subset of LSF object.
+
+        This returns a new GaussianLSF with arrays/attributes that are passed by REFERENCE.
+        This way you can modify the values in the returned object
+        and it will modify the original arrays.
+
+        There are four types of inputs
+        1) integer - returns a specific order
+        2) 1-D slice (single order) - return that slice of the data
+        3) 1-D slice (multiple orders) - return multiple orders using slice
+        4) 1-D slice + integer - returns slice of data for order index
+        """
+
+        # Single index requested
+        if isinstance(index,int):
+            case = 1
+            order = index
+            norder = 1
+            slc = slice(0,self.numpix[order])
+            if index>=self.norder:
+                raise IndexError('Index is too large')
+        # Slice
+        elif isinstance(index,slice):
+            # Single order
+            if self.norder==1:
+                case = 2
+                order = 0
+                norder = 1
+                slc = np.arange(self.numpix[order])[index]
+                #   this ensures that the slice returns the minimum size                
+            # Multiple orders
+            else:
+                case = 3
+                order = index
+                oindex = np.arange(self.norder)[order]
+                norder = len(oindex)
+                slc = slice(None,None,None)  # all
+                if norder==0:
+                    raise IndexError('Order index out of bounds for size '+str(self.norder))
+                # Want all of the orders
+                if norder==self.norder:
+                    return self
+        # Slice + index
+        elif isinstance(index,tuple):
+            case = 4
+            if isinstance(index[0],slice)==False or isinstance(index[1],int)==False:
+                raise IndexError('Must be slice + order index')
+            order = index[1]
+            slc = np.arange(self.numpix[order])[index[0]]
+            #   this ensures that the slice returns the minimum size
+            norder = 1
+        else:
+            raise IndexError('Index must be an integer, slice, or slice+integer')
+
+        kwargs = {}
+        if hasattr(self,'wave') and self.wave is not None:
+            if self.norder==1:
+                kwargs['wave'] = self.wave[slc]
+            else:
+                kwargs['wave'] = self.wave[slc,order]
+            if norder==1: kwargs['wave'] = kwargs['wave'].flatten()
+        if hasattr(self,'pars') and self.pars is not None:
+            if self.norder==1:
+                if np.array(self.pars).ndim==2:
+                    kwargs['pars'] = self.pars[:,0]
+                else:
+                    kwargs['pars'] = self.pars
+            else:
+                kwargs['pars'] = self.pars[:,order]
+        # Scalars
+        for c in ['xtype','lsftype']:
+            kwargs[c] = getattr(self,c)
+        # Make the LSF   
+        olsf = GaussianLsf(**kwargs)
+        # Adding _sigma
+        if hasattr(self,'_sigma') and self._sigma is not None:
+            if self.norder==1:
+                olsf._sigma = self._sigma[slc]
+            else:
+                olsf._sigma = self._sigma[slc,order]
+            if norder==1: olsf._sigma = olsf._sigma.flatten()
+
+        # Single-order pixel slicing, need to fix pars
+        #   only if xtype=pixels and we have pars
+        if (case==2 or case==4) and self.xtype.lower().find('pix')>-1 and hasattr(self,'pars'):
+            sigma1 = self.sigma(order=order)[slc]
+            x = np.arange(olsf.npix)
+            coef = np.polyfit(x,sigma1,len(self.pars.shape[:,0])-1)
+            olsf.pars[:,0] = coef[::-1]
+            
+        return olsf
+
+                
 # Class for representing Gauss-Hermite LSFs
 class GaussHermiteLsf(Lsf):
     """
@@ -1497,7 +1919,7 @@ class GaussHermiteLsf(Lsf):
             raise Exception("No LSF parameters")
         # Get parameters
         pars = self.pars
-        if self.ndim==2: pars=self.pars[:,order]
+        if pars.ndim==2: pars=self.pars[:,order]
         # Wavelengths input
         if xtype.lower().find('wave') > -1:
             w = x.copy()
@@ -1508,15 +1930,13 @@ class GaussHermiteLsf(Lsf):
         poly = np.polynomial.Polynomial(params['GHcoefs'][0])
         sigma = poly(x+params['Xoffset'])
         return sigma
-
     
     # Clean up bad LSF values
     def clean(self):
         """ Clean the LSF values.  This is not implemented for Gauss-Hermite LSF."""
         # No cleaning for now
         pass
-
-    
+   
     # Return full LSF values for the spectrum
     def array(self,order=None):
         """
@@ -1641,3 +2061,244 @@ class GaussHermiteLsf(Lsf):
         # right, need to multiply by dx.
             
         return lsf
+
+    def trim(self,w0,w1):
+        """
+        Trim the LSF in wavelength.
+
+        Parameters
+        ----------
+        w0 : float
+           Lower wavelength (in Ang) to trim.
+        w1 : float
+           Upper wavelength (in Ang) to trim.
+
+        Returns
+        -------
+        Nothing is returned.  The spectrum is trimmed in place.
+
+        Examples
+        --------
+
+        lsf.trim(5500.0,6500.0)
+
+        """
+        # Get number of pixels per order
+        ranges = np.zeros([self.norder,2],int)-1
+        ngood = np.zeros(self.norder,int)        
+        for o in range(self.norder):
+            if self.norder==1:
+                ind, = np.where((self.wave>=w0) & (self.wave<=w1))
+            else:
+                npix = self.numpix[o]
+                ind, = np.where((self.wave[:npix,o]>=w0) & (self.wave[:npix,o]<=w1))
+            ngood[o] = len(ind)
+            if len(ind)>0:
+                ranges[o,0] = ind[0]
+                ranges[o,1] = ind[-1]
+        # Check that we have some pixels
+        goodorder, = np.where(ngood>0)
+        if len(goodorder)==0:
+            raise ValueError('No pixels left')
+        # Remove blank orders
+        blankorder, = np.where(ngood==0)
+        for o in blankorder:
+            self.remove_order(o)
+        ranges = ranges[goodorder,:]
+        numpix = list(ranges[:,1]-ranges[:,0]+1)
+        npix = np.max(numpix)    # new npix  
+        if len(goodorder)==1:
+            ranges = ranges.flatten()
+        # Trim arrays
+        default = {'wave':0.0,'_sigma':0.0}
+        for c in ['wave','_sigma']:
+            if hasattr(self,c) and getattr(self,c) is not None:
+                arr = getattr(self,c)
+                if self.norder==1:
+                    newarr = arr[ranges[0]:ranges[1]+1]
+                else:
+                    newarr = np.zeros([npix,self.norder],arr.dtype)
+                    newarr[:,:] = default[c]
+                    for o in range(self.norder):
+                        newvals = arr[ranges[o,0]:ranges[o,1]+1,o].copy()
+                        newarr[:len(newvals),o] = newvals
+                setattr(self,c,newarr)
+        self.numpix = numpix
+        # Fix pars if using pixel-based values
+        #   xtype='Wave' is fine
+        if self.xtype.lower().find('pix')>-1:
+            # Just shift the Xshift parameter
+            if self.norder==1:
+                self.pars[1,0] += ranges[0]
+            else:
+                for o in range(self.norder):
+                    self.pars[1,o] += ranges[o,0]
+
+    def rebin(self,nbin,mean=False):
+        """
+        Rebin the LSF information in the spectral dimension.
+
+        Parameters
+        ----------
+        nbin : int
+            Integer number of pixels to bin.
+        mean : bool, optional
+            Take the mean instead of the default sum.
+
+        Returns
+        -------
+        Nothing is returned.  The spectrum is rebinned in place.
+
+        Examples
+        --------
+
+        lsf.rebin(2)
+
+        """
+        # Only keep complete bins and remove any excess
+        newnpix = self.npix//nbin
+        numpix = self.numpix.copy()
+        oldnumpix = self.numpix.copy()
+        # Bin the data
+        default = {'wave':0.0,'_sigma':0.0}
+        for c in ['wave','_sigma']:
+            if hasattr(self,c) and getattr(self,c) is not None:
+                arr = getattr(self,c)
+                if self.norder==1:
+                    arr = np.atleast_2d(arr).T
+                # Use mean for both wave and _sigma
+                newarr = np.zeros([newnpix,self.norder],arr.dtype)
+                newarr[:,:] = default[c]
+                for o in range(self.norder):
+                    arr1 = arr[:self.numpix[o],o]  # trim to good values
+                    newvals = dln.rebin(arr1,binsize=nbin)
+                    if c=='_sigma' and self.xtype.lower().find('pix')>-1:
+                        newvals /= nbin    # reduce for new binsize
+                    newarr[:len(newvals),o] = newvals
+                    numpix[o] = len(newvals)
+                if self.norder==1:
+                    newarr = newarr.flatten()
+                setattr(self,c,newarr)
+        self.numpix = numpix
+        # Fix pars if using pixel-based values
+        #   xtype='Wave' is fine
+        if self.xtype.lower().find('pix')>-1:
+            # apogee_drp apdithercomb.pro does something similar
+            pardict = unpack_ghlsf_params(self.pars)            
+            for o in range(self.norder):
+                params = pardict[o]
+                newparams = params.copy()
+                newparams['binsize'] *= nbin
+                newparams['Xoffset'] /= nbin
+                # The Gauss-Hermite coefficients:
+                # [sigma, H0, H1, H2, H3, H4, ...] up to H9.
+                # Since we're changing X -> X/nbin we need to scale the
+                # polynomial coefficients depending on their power
+                # c0 -> c0
+                # c1 -> c1*nbin
+                # c2 -> c2*nbin**2   and so on
+                # GHcoefs is [GH parameter, Polynomial order]
+                #  skip 1st, constant them
+                for i in np.arange(1,np.max(params['Porder'])+1):
+                    # change all GH parameters of the ith order
+                    #   if they are zero, nothing will cahnge
+                    newparams['GHcoefs'][:,i] *= nbin**i
+                # Reduce GH sigma (1st parameter) since our pixels are getting larger
+                newparams['GHcoefs'][0,:] /= nbin
+                # Same for wing parameters
+                for i in np.arange(1,np.max(params['WPorder'])+1):
+                    newparams['Wcoefs'][:,i] *= nbin**i
+                # Reduce wing sigma (1st parameter) since our pixels are getting larger
+                newparams['Wcoefs'][0,:] /= nbin
+                pardict[o] = newparams
+            lsfarr = repack_ghlsf_params(pardict)
+            self.pars = lsfarr
+                
+    def __getitem__(self,index):
+        """ 
+        Return slice or subset of LSF object.
+
+        This returns a new GaussHermiteLSF with arrays/attributes that are passed by REFERENCE.
+        This way you can modify the values in the returned object
+        and it will modify the original arrays.
+
+        There are four types of inputs
+        1) integer - returns a specific order
+        2) 1-D slice (single order) - return that slice of the data
+        3) 1-D slice (multiple orders) - return multiple orders using slice
+        4) 1-D slice + integer - returns slice of data for order index
+        """
+
+        # Single index requested
+        if isinstance(index,int):
+            case = 1
+            order = index
+            norder = 1
+            slc = slice(0,self.numpix[order])
+            if index>=self.norder:
+                raise IndexError('Index is too large')
+        # Slice
+        elif isinstance(index,slice):
+            # Single order
+            if self.norder==1:
+                case = 2
+                order = 0
+                norder = 1
+                slc = np.arange(self.numpix[order])[index]
+                #   this ensures that the slice returns the minimum size                
+            # Multiple orders
+            else:
+                case = 3
+                order = index
+                oindex = np.arange(self.norder)[order]
+                norder = len(oindex)
+                slc = slice(None,None,None)  # all
+                if norder==0:
+                    raise IndexError('Order index out of bounds for size '+str(self.norder))
+                # Want all of the orders
+                if norder==self.norder:
+                    return self
+        # Slice + index
+        elif isinstance(index,tuple):
+            case = 4
+            if isinstance(index[0],slice)==False or isinstance(index[1],int)==False:
+                raise IndexError('Must be slice + order index')
+            order = index[1]
+            slc = np.arange(self.numpix[order])[index[0]]
+            #   this ensures that the slice returns the minimum size
+            norder = 1
+        else:
+            raise IndexError('Index must be an integer, slice, or slice+integer')
+
+        kwargs = {}
+        if hasattr(self,'wave') and self.wave is not None:
+            if self.norder==1:
+                kwargs['wave'] = self.wave[slc]
+            else:
+                kwargs['wave'] = self.wave[slc,order]
+            if norder==1: kwargs['wave'] = kwargs['wave'].flatten()
+        if hasattr(self,'pars') and self.pars is not None:
+            if self.norder==1:
+                kwargs['pars'] = self.pars[:].copy()
+            else:
+                kwargs['pars'] = self.pars[:,order].copy()
+        # Scalars
+        for c in ['xtype','lsftype']:
+            kwargs[c] = getattr(self,c)
+        # Make the LSF   
+        olsf = GaussHermiteLsf(**kwargs)
+        # Adding _sigma
+        if hasattr(self,'_sigma') and self._sigma is not None:
+            if self.norder==1:
+                olsf._sigma = self._sigma[slc]
+            else:
+                olsf._sigma = self._sigma[slc,order]
+            if norder==1: olsf._sigma = olsf._sigma.flatten()
+        
+        # Single-order pixel slicing, need to fix pars
+        #   only if xtype=pixels and we have pars
+        if (case==2 or case==4) and self.xtype.lower().find('pix')>-1 and hasattr(self,'pars'):
+            # Just shift the Xshift parameter
+            olsf.pars[1,0] += slc[0]
+            
+        return olsf
