@@ -146,7 +146,7 @@ def read(filename=None,format=None):
             out = spec1d(filename)
             datacheck(out)
             if out is not None: return out
-        
+            
     # Loop over all readers until we get a spectrum out
     for k in _readers.keys():
         try:
@@ -348,12 +348,16 @@ def apvisit(filename):
     x = np.arange(spec.npix)
     nsky = 4
     for i in range(spec.norder):
-        sky = spec.sky[:,i]
-        medsky = median_filter(sky,201,mode='reflect')
-        medcoef = dln.poly_fit(x,medsky/np.median(medsky),2)
-        medsky2 = dln.poly(x,medcoef)*np.median(medsky)
-        skymask1 = (sky>nsky*medsky2)    # pixels Nsig above median sky
-        mask[:,i] = np.logical_or(mask[:,i],skymask1)    # OR combine
+        try:
+            sky = spec.sky[:,i]
+            medsky = median_filter(sky,201,mode='reflect')
+            good = np.isfinite(edsky/np.nanmedian(medsky))
+            medcoef = dln.poly_fit(x[good],medsky[good]/np.nanmedian(medsky[good]),2)
+            medsky2 = dln.poly(x,medcoef)*np.nanmedian(medsky)
+            skymask1 = (sky>nsky*medsky2)    # pixels Nsig above median sky
+            mask[:,i] = np.logical_or(mask[:,i],skymask1)    # OR combine
+        except:
+            pass
     spec.mask = mask
     # Fix NaN pixels
     for i in range(spec.norder):
@@ -400,16 +404,18 @@ def apstar(filename):
 
     # Get number of extensions
     hdulist = fits.open(filename)
+    header = hdulist[0].header
     nhdu = len(hdulist)
-    hdulist.close()
+
     
     # Check that this has the right format
     validfile = False
     if (base.find("apStar") == -1) | (base.find("asStar") == -1):
         if nhdu>=9:
-            gd, = np.where(np.char.array(hdulist[0].header['HISTORY']).astype(str).find('APSTAR') > -1)
-            if len(gd)>0: validfile=True
+            if header.get('starflag') is not None and header.get('objid') is not None:
+                validfile = True
     if validfile is False:
+        hdu.close()
         return None
     
     
@@ -439,32 +445,34 @@ def apstar(filename):
     #  these are 2D arrays with [Nvisit+2,Npix]
     #  the first two are combined and the rest are the individual spectra
 
-    head1 = fits.getheader(filename,1)
+    head1 = hdulist[1].header
+    #head1 = fits.getheader(filename,1)
     w0 = np.float64(head1["CRVAL1"])
     dw = np.float64(head1["CDELT1"])
     nw = head1["NAXIS1"]
     wave = 10**(np.arange(nw)*dw+w0)
-        
+
+    # ONLY load the FIRST combined spectrum
+    
     # flux, err, sky, skyerr are in units of 1e-17
-    flux = fits.getdata(filename,1).T * 1e-17
-    lsfcoef = fits.getdata(filename,8).T
+    flux = hdulist[1].data[0] * 1e-17
+    lsfcoef = hdulist[8].data
     spec = Spec1D(flux,wave=wave,lsfpars=lsfcoef,lsftype='Gauss-Hermite',lsfxtype='Pixels')
     spec.reader = 'apstar'
     spec.filename = filename
     spec.sptype = "apStar"
     spec.waveregime = "NIR"
     spec.instrument = "APOGEE"
-    spec.head = fits.getheader(filename,0)
-    spec.err = fits.getdata(filename,2).T * 1e-17
+    spec.head = header
+    spec.err = hdulist[2].data[0] * 1e-17
     #bad = (spec.err<=0)   # fix bad error values
     #if np.sum(bad) > 0:
     #    spec.err[bad] = 1e30
-    spec.bitmask = fits.getdata(filename,3)
-    spec.sky = fits.getdata(filename,4).T * 1e-17
-    spec.skyerr = fits.getdata(filename,5).T * 1e-17
-    spec.telluric = fits.getdata(filename,6).T
-    spec.telerr = fits.getdata(filename,7).T
-    spec.lsf = fits.getdata(filename,8).T
+    spec.bitmask = hdulist[3].data[0]
+    spec.sky = hdulist[4].data[0] * 1e-17
+    spec.skyerr = hdulist[5].data[0] * 1e-17
+    spec.telluric = hdulist[6].data[0]
+    spec.telerr = hdulist[7].data[0]
     # Create the bad pixel mask
     # "bad" pixels:
     #   flag = ['BADPIX','CRPIX','SATPIX','UNFIXABLE','BADDARK','BADFLAT','BADERR','NOSKY',
@@ -476,25 +484,31 @@ def apstar(filename):
     x = np.arange(spec.npix)
     nsky = 4
     medsky = median_filter(spec.sky,201,mode='reflect')
-    medcoef = dln.poly_fit(x,medsky/np.median(medsky),2)
+    medsky[~np.isfinite(medsky)] = 0.0
+    mmedsky = np.nanmedian(medsky)
+    if mmedsky==0:
+        mmedsky = 1.0
+    medcoef = dln.poly_fit(x,(medsky/mmedsky),2)
     medsky2 = dln.poly(x,medcoef)*np.median(medsky)
-    skymask1 = (sky>nsky*medsky2)    # pixels Nsig above median sky
-    mask[:,i] = np.logical_or(mask[:,i],skymask1)    # OR combine
+    skymask1 = (spec.sky>nsky*medsky2)     # pixels Nsig above median sky
+    mask = np.logical_or(mask,skymask1)    # OR combine
     spec.mask = mask
     # Fix NaN or bad pixels pixels
-    bd,nbd = dln.where( (np.isfinite(spec.flux[:,i])==False) | (spec.err[:,i] <= 0.0) )
+    bd,nbd = dln.where( (np.isfinite(spec.flux)==False) | (spec.err <= 0.0) )
     if nbd>0:
         spec.flux[bd] = 0.0
         spec.err[bd] = 1e30
         spec.mask[bd] = True
     if nhdu>=9:
-        spec.meta = fits.getdata(filename,9)    # meta-data
-    #spec.snr = spec.head["SNR"]
+        spec.meta = hdulist[9].data    # meta-data
     if base.find("apStar") > -1:
         spec.observatory = 'apo'
     else:
         spec.observatory = 'lco'
-    spec.wavevac = True            
+    spec.wavevac = True
+
+    hdulist.close()
+    
     return spec
 
 
@@ -543,10 +557,24 @@ def boss(filename):
     # HDU3 - table with line measurements
     head = fits.getheader(filename,0)
     tab1 = Table.read(filename,1)
+    for c in tab1.colnames:   # Make column names all lowercase
+        tab1[c].name=c.lower()
     cat1 = Table.read(filename,2)
+    for c in cat1.colnames:   # Make column names all lowercase                            
+        cat1[c].name=c.lower()
     flux = tab1["flux"].data
     wave = 10**tab1["loglam"].data
     wdisp = tab1["wdisp"].data
+    lsfsigma = wdisp
+    lsfxtype = 'Pixels'
+    # Use WRESL by default
+    if 'wresl' in tab1.colnames:
+        gd, = np.where((tab1['wresl'].data > 0) & np.isfinite(tab1['wresl'].data))
+        if len(gd)==0:
+            print('No good values in WRESL. Using WDISP instead')
+        else:
+            lsfsigma = tab1["wresl"].data / 2.35
+            lsfxtype = 'Wave'
     # checking for zeros in IVAR
     ivar = tab1["ivar"].data.copy()
     bad = (ivar<=0)
@@ -559,23 +587,28 @@ def boss(filename):
     else:
         err = 1.0/np.sqrt(ivar)
         mask = np.zeros(flux.shape,bool)
-    spec = Spec1D(flux,err=err,wave=wave,mask=mask,lsfsigma=wdisp,lsfxtype='Wave')
+    spec = Spec1D(flux,err=err,wave=wave,mask=mask,lsfsigma=lsfsigma,lsfxtype=lsfxtype)
     spec.reader = 'boss'
     spec.lsf.clean()   # clean up some bad LSF values
     spec.filename = filename
     spec.sptype = "spec"
     spec.waveregime = "Optical"
     spec.instrument = "BOSS"
+    if head.get('DATE-OBS') is None:
+        if head.get('INTSTART') is not None:
+            head['DATE-OBS'] = head['INTSTART']
     spec.head = head
     spec.ivar = tab1["ivar"].data
     spec.bitmask = tab1["or_mask"].data
     spec.and_mask = tab1["and_mask"].data
     spec.or_mask = tab1["or_mask"].data
     spec.sky = tab1["sky"].data
-    spec.model = tab1["model"].data
+    if 'model' in np.char.array(tab1.colnames).lower():
+        spec.model = tab1["model"].data
     spec.meta = cat1
     # What are the units?
-    #spec.snr = cat1["SN_MEDIAN_ALL"].data[0]
+    if 'sn_median_all' in np.char.array(tab1.colnames).lower():
+        spec.snr = cat1["sn_median_all"].data[0]
     spec.observatory = 'apo'
     spec.wavevac = True
     return spec        
@@ -865,7 +898,7 @@ def iraf(filename):
     cdelt1 = head.get('cdelt1')
     if cdelt1 is None: cdelt1=head.get('cd1_1')
     if (crval1 is not None) & (crpix1 is not None) & (cdelt1 is not None):
-        wave = (np.arange(npix,int)+1-crpix1) * np.float64(cdelt1) + np.float64(crval1)
+        wave = (np.arange(npix)+1-crpix1) * np.float64(cdelt1) + np.float64(crval1)
         wlogflag = head.get('DC-FLAG')
         if (wlogflag is not None):
             if wlogflag==1: wave = 10**wave
@@ -885,7 +918,14 @@ def iraf(filename):
             if 'sigma' in val:
                 if (i<=norder):
                     spec.err = data[:,k]
-
+    if spec.err is None:
+        spec.err = np.sqrt(spec.flux)
+        bad = (spec.err <= 0)
+        if np.sum(bad)>0:        
+            good = (spec.err > 0)
+            minerr = np.minimum(np.min(spec.err[good]),1)
+            spec.err[bad] = minerr
+                    
     return spec
 
 # List of readers
